@@ -1,5 +1,4 @@
 from .jupyters3 import JupyterS3
 from .jupyters3 import JupyterS3SecretAccessKeyAuthentication
 from .jupyters3 import JupyterS3ECSRoleAuthentication
-from .jupyters3 import AwsCreds
-from .jupyters3 import JupyterS3Authentication
+from .jupyters3 import JupyterS3EC2RoleAuthentication
