@@ -44,7 +44,7 @@ NOTEBOOK_SUFFIX = '.ipynb'
 CHECKPOINT_SUFFIX = '.checkpoints'
 UNTITLED_NOTEBOOK = 'Untitled'
 UNTITLED_FILE = 'Untitled'
-UNTITLED_DIRECTORY  = 'Untitled Folder'
+UNTITLED_DIRECTORY = 'Untitled Folder'
 
 Context = namedtuple('Context', [
     'logger', 'prefix', 'region', 's3_bucket', 's3_host', 's3_auth',
@@ -110,6 +110,37 @@ class JupyterS3SecretAccessKeyAuthentication(JupyterS3Authentication):
         )
 
 
+class JupyterS3EC2RoleAuthentication(JupyterS3Authentication):
+
+    role_name = Unicode(config=True)
+    aws_access_key_id = Unicode()
+    aws_secret_access_key = Unicode()
+    pre_auth_headers = Dict()
+    expiration = Datetime()
+
+    @gen.coroutine
+    def get_credentials(self):
+        now = datetime.datetime.now()
+
+        if now > self.expiration:
+            request = HTTPRequest(
+                'http://169.254.169.254/latest/meta-data/iam/security-credentials/' + self.role_name, method='GET')
+            creds = json.loads((yield AsyncHTTPClient().fetch(request)).body.decode('utf-8'))
+            self.aws_access_key_id = creds['AccessKeyId']
+            self.aws_secret_access_key = creds['SecretAccessKey']
+            self.pre_auth_headers = {
+                'x-amz-security-token': creds['Token'],
+            }
+            self.expiration = datetime.datetime.strptime(
+                creds['Expiration'], '%Y-%m-%dT%H:%M:%SZ')
+
+        return AwsCreds(
+            access_key_id=self.aws_access_key_id,
+            secret_access_key=self.aws_secret_access_key,
+            pre_auth_headers=self.pre_auth_headers,
+        )
+
+
 class JupyterS3ECSRoleAuthentication(JupyterS3Authentication):
 
     aws_access_key_id = Unicode()
@@ -122,14 +153,16 @@ class JupyterS3ECSRoleAuthentication(JupyterS3Authentication):
         now = datetime.datetime.now()
 
         if now > self.expiration:
-            request = HTTPRequest('http://169.254.170.2' + os.environ['AWS_CONTAINER_CREDENTIALS_RELATIVE_URI'], method='GET')
+            request = HTTPRequest(
+                'http://169.254.170.2' + os.environ['AWS_CONTAINER_CREDENTIALS_RELATIVE_URI'], method='GET')
             creds = json.loads((yield AsyncHTTPClient().fetch(request)).body.decode('utf-8'))
             self.aws_access_key_id = creds['AccessKeyId']
             self.aws_secret_access_key = creds['SecretAccessKey']
             self.pre_auth_headers = {
                 'x-amz-security-token': creds['Token'],
             }
-            self.expiration = datetime.datetime.strptime(creds['Expiration'], '%Y-%m-%dT%H:%M:%SZ')
+            self.expiration = datetime.datetime.strptime(
+                creds['Expiration'], '%Y-%m-%dT%H:%M:%SZ')
 
         return AwsCreds(
             access_key_id=self.aws_access_key_id,
@@ -333,7 +366,8 @@ def _file_exists(context, path):
             response = yield _make_s3_request(context, 'HEAD', '/' + key, {}, {}, b'')
         except HTTPClientError as exception:
             if exception.response.code != 404:
-                raise HTTPServerError(exception.response.code, 'Error checking if S3 exists')
+                raise HTTPServerError(
+                    exception.response.code, 'Error checking if S3 exists')
             response = exception.response
 
         return response.code == 200
@@ -343,13 +377,14 @@ def _file_exists(context, path):
 
 @gen.coroutine
 def _exists(context, path):
-    return (yield _file_exists(context, path)) or (yield _dir_exists(context, path)) 
+    return (yield _file_exists(context, path)) or (yield _dir_exists(context, path))
 
 
 @gen.coroutine
 def _get(context, path, content, type, format):
     type_to_get = type if type is not None else (yield _type_from_path(context, path))
-    format_to_get = format if format is not None else _format_from_type_and_path(context, type_to_get, path)
+    format_to_get = format if format is not None else _format_from_type_and_path(
+        context, type_to_get, path)
     return (yield GETTERS[(type_to_get, format_to_get)](context, path, content))
 
 
@@ -376,18 +411,19 @@ def _get_any(context, path, content, type, mimetype, format, decode):
     response = yield _make_s3_request(context, method, '/' + key, {}, {}, b'')
     file_bytes = response.body
     last_modified_str = response.headers['Last-Modified']
-    last_modified = datetime.datetime.strptime(last_modified_str, "%a, %d %b %Y %H:%M:%S GMT")
+    last_modified = datetime.datetime.strptime(
+        last_modified_str, "%a, %d %b %Y %H:%M:%S GMT")
     return {
         'name': _final_path_component(path),
         'path': path,
         'type': type,
         'mimetype': mimetype,
         'writable': True,
-        'last_modified': last_modified, 
+        'last_modified': last_modified,
         'created': last_modified,
-        'format': format if content else None,  
+        'format': format if content else None,
         'content': decode(file_bytes) if content else None,
-    }  
+    }
 
 
 @gen.coroutine
@@ -406,7 +442,7 @@ def _get_directory(context, path, content):
         'type': 'directory',
         'mimetype': None,
         'writable': True,
-        'last_modified': datetime.datetime.fromtimestamp(86400), 
+        'last_modified': datetime.datetime.fromtimestamp(86400),
         'created': datetime.datetime.fromtimestamp(86400),
         'format': 'json' if content else None,
         'content': ([
@@ -433,7 +469,8 @@ def _get_directory(context, path, content):
 @gen.coroutine
 def _save(context, model, path):
     type_to_save = model['type'] if 'type' in model else (yield _type_from_path(context, path))
-    format_to_save = model['format'] if 'format' in model else _format_from_type_and_path(context, type_to_save, path)
+    format_to_save = model['format'] if 'format' in model else _format_from_type_and_path(
+        context, type_to_save, path)
     return (yield SAVERS[(type_to_save, format_to_save)](
         context,
         model['chunk'] if 'chunk' in model else None,
@@ -493,7 +530,8 @@ def _save_bytes(context, content_bytes, path, type, mimetype):
     response = yield _make_s3_request(context, 'PUT', '/' + key, {}, {}, content_bytes)
 
     last_modified_str = response.headers['Date']
-    last_modified = datetime.datetime.strptime(last_modified_str, "%a, %d %b %Y %H:%M:%S GMT")
+    last_modified = datetime.datetime.strptime(
+        last_modified_str, "%a, %d %b %Y %H:%M:%S GMT")
     return _saved_model(path, type, mimetype, last_modified)
 
 
@@ -504,7 +542,7 @@ def _saved_model(path, type, mimetype, last_modified):
         'type': type,
         'mimetype': mimetype,
         'writable': True,
-        'last_modified': last_modified, 
+        'last_modified': last_modified,
         'created': last_modified,
         'format': None,
         'content': None,
@@ -759,7 +797,7 @@ def _list_keys(context, key_prefix, delimeter):
     @gen.coroutine
     def _list_later_page(token):
         query = {
-             **common_query,
+            **common_query,
             'continuation-token': token,
         }
         response = yield _make_s3_request(context, 'GET', '/', query, {}, b'')
@@ -779,12 +817,15 @@ def _list_keys(context, key_prefix, delimeter):
         for el in root:
             if el.tag == f'{namespace}Contents':
                 key = _first_child_text(el, f'{namespace}Key')
-                last_modified_str = _first_child_text(el, f'{namespace}LastModified')
-                last_modified = datetime.datetime.strptime(last_modified_str, "%Y-%m-%dT%H:%M:%S.%fZ")
+                last_modified_str = _first_child_text(
+                    el, f'{namespace}LastModified')
+                last_modified = datetime.datetime.strptime(
+                    last_modified_str, "%Y-%m-%dT%H:%M:%S.%fZ")
                 keys.append((key, last_modified))
             if el.tag == f'{namespace}CommonPrefixes':
                 # Prefixes end in '/', which we strip off
-                directories.append(_first_child_text(el, f'{namespace}Prefix')[:-1])
+                directories.append(_first_child_text(
+                    el, f'{namespace}Prefix')[:-1])
             if el.tag == f'{namespace}NextContinuationToken':
                 next_token = el.text
 
@@ -813,9 +854,11 @@ def _make_s3_request(context, method, path, query, api_pre_auth_headers, payload
         service, context.region, context.s3_host, method, full_path, query, payload,
     )
 
-    querystring = urllib.parse.urlencode(query, safe='~', quote_via=urllib.parse.quote)
+    querystring = urllib.parse.urlencode(
+        query, safe='~', quote_via=urllib.parse.quote)
     encoded_path = urllib.parse.quote(full_path, safe='/~')
-    url = f'https://{context.s3_host}{encoded_path}' + (('?' + querystring) if querystring else '')
+    url = f'https://{context.s3_host}{encoded_path}' + \
+        (('?' + querystring) if querystring else '')
 
     body = \
         payload if method == 'PUT' else \
@@ -859,11 +902,14 @@ def _aws_sig_v4_headers(access_key_id, secret_access_key, pre_auth_headers,
         def canonical_request():
             canonical_uri = urllib.parse.quote(path, safe='/~')
             quoted_query = sorted(
-                (urllib.parse.quote(key, safe='~'), urllib.parse.quote(value, safe='~'))
+                (urllib.parse.quote(key, safe='~'),
+                 urllib.parse.quote(value, safe='~'))
                 for key, value in query.items()
             )
-            canonical_querystring = '&'.join(f'{key}={value}' for key, value in quoted_query)
-            canonical_headers = ''.join(f'{key}:{headers[key]}\n' for key in header_keys)
+            canonical_querystring = '&'.join(
+                f'{key}={value}' for key, value in quoted_query)
+            canonical_headers = ''.join(
+                f'{key}:{headers[key]}\n' for key in header_keys)
 
             return f'{method}\n{canonical_uri}\n{canonical_querystring}\n' + \
                    f'{canonical_headers}\n{signed_headers}\n{payload_hash}'
@@ -872,9 +918,11 @@ def _aws_sig_v4_headers(access_key_id, secret_access_key, pre_auth_headers,
             return hmac.new(key, msg.encode('utf-8'), hashlib.sha256).digest()
 
         string_to_sign = f'{algorithm}\n{amzdate}\n{credential_scope}\n' + \
-                         hashlib.sha256(canonical_request().encode('utf-8')).hexdigest()
+                         hashlib.sha256(
+                             canonical_request().encode('utf-8')).hexdigest()
 
-        date_key = sign(('AWS4' + secret_access_key).encode('utf-8'), datestamp)
+        date_key = sign(
+            ('AWS4' + secret_access_key).encode('utf-8'), datestamp)
         region_key = sign(date_key, region)
         service_key = sign(region_key, service)
         request_key = sign(service_key, 'aws4_request')
@@ -885,13 +933,15 @@ def _aws_sig_v4_headers(access_key_id, secret_access_key, pre_auth_headers,
         'x-amz-date': amzdate,
         'x-amz-content-sha256': payload_hash,
         'Authorization': f'{algorithm} Credential={access_key_id}/{credential_scope}, '
-                         f'SignedHeaders={signed_headers}, Signature=' + signature(),
+                         f'SignedHeaders={signed_headers}, Signature=' +
+        signature(),
     }
 
 
 def _run_sync_in_new_thread(func):
     result = None
     exception = None
+
     def _func():
         nonlocal result
         nonlocal exception
